﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Telerik.Windows.Controls;
using System.Collections;
using DeleteFromEntitySet.Web;
using System.Collections.ObjectModel;
using Telerik.Windows.Data;
using System.ServiceModel.DomainServices.Client;
using System.Collections.Specialized;
using System.ComponentModel;
using Microsoft.Practices.Composite.Presentation.Commands;

namespace DeleteFromEntitySet
{
    public class MyViewModel : ViewModelBase
    {
        private readonly ChinookContext context;
        private RadObservableCollection<Artist> artists;
        
        public MyViewModel()
        {
            this.context = new ChinookContext();
            this.context.PropertyChanged += OnContextPropertyChanged;

            this.SubmitCommand = new ActionCommand(() => this.context.SubmitChanges(), () => this.context.HasChanges);
        }

        void OnContextPropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "IsLoading")
            {
                this.IsLoading = this.context.IsLoading;
            }
            else if (e.PropertyName == "HasChanges")
            {
                this.SubmitCommand.RaiseCanExecuteChanged();
            }
        }

        public ActionCommand SubmitCommand
        {
            get;
            private set;
        }

        public IEnumerable Artists 
        {
            get
            {
                if (this.artists == null)
                {
                    this.artists = new RadObservableCollection<Artist>();

                    this.context.Load(this.context.GetArtistsQuery(), LoadArtistsCompleted, null);
                }

                return artists;
            }
        }

        private void LoadArtistsCompleted(LoadOperation<Artist> loadOperation)
        {
            this.artists.AddRange(loadOperation.Entities);
            this.artists.CollectionChanged += OnArtistsCollectionChanged;

            foreach (var artist in this.artists)
            {
                artist.AlbumsChanged += OnArtistAlbumsChanged;
            }
        }

        void OnArtistAlbumsChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            switch (e.Action)
            {
                case NotifyCollectionChangedAction.Add:
                    this.context.Albums.Add(e.NewItems[0] as Album);
                    break;
                case NotifyCollectionChangedAction.Remove:
                    this.context.Albums.Remove(e.OldItems[0] as Album);
                    break;
                case NotifyCollectionChangedAction.Replace:
                    this.context.Albums.Add(e.NewItems[0] as Album);
                    this.context.Albums.Remove(e.OldItems[0] as Album);
                    break;
                case NotifyCollectionChangedAction.Reset:
                default:
                    break;
            }
        }

        void OnArtistsCollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            switch (e.Action)
            {
                case NotifyCollectionChangedAction.Add:
                    this.AddArtist(e.NewItems[0] as Artist);
                    break;
                case NotifyCollectionChangedAction.Remove:
                    this.RemoveArtist(e.OldItems[0] as Artist);
                    break;
                case NotifyCollectionChangedAction.Replace:
                    this.AddArtist(e.NewItems[0] as Artist);
                    this.RemoveArtist(e.OldItems[0] as Artist);
                    break;
                case NotifyCollectionChangedAction.Reset:
                default:
                    break;
            }
        }


        private void AddArtist(Artist newArtist)
        {
            this.context.Artists.Add(newArtist);
            newArtist.AlbumsChanged += OnArtistAlbumsChanged;
        }

        private void RemoveArtist(Artist oldArtist)
        {
            this.context.Artists.Remove(oldArtist);
            oldArtist.AlbumsChanged -= OnArtistAlbumsChanged;
        }

        private bool isLoading;

        public bool IsLoading
        {
            get
            {
                return this.isLoading;
            }
            private set
            {
                if (this.isLoading != value)
                {
                    this.isLoading = value;
                    this.OnPropertyChanged("IsLoading");
                }
            }
        }
    }
}
